package org.apache.xindice.server;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: FileResource.java,v 1.1.1.1 2001/12/06 19:33:55 bradford Exp $
 */

import java.io.*;

/**
 * FileResource represents and provides a manipulation interface to
 * Xindice's abstracted FileManager.
 */

public interface FileResource {
   /**
    * getInputStream returns an InputStream for reading the
    * FileResource.
    *
    * @return the InputStream
    */
   InputStream getInputStream();

   /**
    * getOutputStream returns an OutputStream for overwriting
    * the FileResource.
    *
    * @return the OutputStream
    */
   OutputStream getOutputStream();

   /**
    * getAppendStream returns an OutputStream for appending
    * to the FileResource.
    *
    * @return the OutputStream.
    */
   OutputStream getAppendStream();

   /**
    * copyStreamTo copies the data of this FileResource to the
    * specified OutputStream.
    *
    * @param os the OutputStream
    */
   void copyStreamTo(OutputStream os) throws IOException;

   /**
    * move moves the file specified by this FileResource to the
    * pathname given by the filename argument.
    *
    * @param filename the new filename
    * @return true if the move succeeds
    */
   boolean move(String filename);

   /**
    * rename renames the file specified by this FileResource to have
    * the pathname given by the filename argument.
    *
    * @param filename the new filename
    * @return true if the renaming succeeds
    */
   boolean rename(String filename);

   /**
    * getChildren returns a list of the files in the directory
    * specified by this FileResource.
    *
    * @return an array of FileResources that are children of this FileResource
    */
   FileResource[] getChildren();

   /**
    * getName returns the name of the file represented by this object.
    * The name is everything in the pathame after the last occurrence of the
    * separator character.
    *
    * @return the name of the file (without any directory components)
    */
   String getName();

   /**
    * getPath returns the pathname of the file represented by this
    * object.
    *
    * @return the pathname represented by this FileResource
    */
   String getPath();

   /**
    * getAbsolutePath returns the absolute pathname of the file
    * represented by this object. If this object represents an absolute
    * pathname, then return the pathname. Otherwise, return a pathname that
    * is a concatenation of the current user directory, the separator
    * character, and the pathname of this FileResource.
    *
    * @return a system-dependent absolute pathname for this File
    */
   String getAbsolutePath();

   /**
    * getCanonicalPath Returns the canonical form of this File
    * object's pathname. The precise definition of canonical form is
    * system-dependent, but it usually specifies an absolute pathname in
    * which all relative references and references to the current user
    * directory have been completely resolved. The canonical form of a
    * pathname of a nonexistent file may not be defined.
    *
    * @return a system-dependent absolute pathname for this FileResource
    * @throws IOException If an I/O error occurs
    */
   String getCanonicalPath() throws IOException;

   /**
    * exists tests if this File exists
    *
    * @return true if the file specified by this object exists
    */
   boolean exists();

   /**
    * isFile tests if the file represented by this FileResource is a
    * "normal" file.
    *
    * A file is "normal" if it is not a directory and, in addition, satisfies
    * other system-dependent criteria. Any non-directory file created by a
    * Java application is guaranteed to be a normal file.
    *
    * @return true if the file specified by this object exists and is a "normal" file
    */
   boolean isFile();

   /**
    * isDirectory tests if the file represented by this FileResource
    * is a directory.
    *
    * @return true if this File exists and is a directory
    */
   boolean isDirectory();

   /**
    * lastModified Returns the time that the file represented by this
    * FileResource was last modified.  The return value is system dependent and
    * should only be used to compare with other values returned by last
    * modified. It should not be interpreted as an absolute time.
    *
    * @return the time the file specified by this object was last modified
    */
   long lastModified();

   /**
    * length returns the length of the file represented by this File
    * object.
    *
    * @return the length, in bytes, of the file specified by this object
    */
   long length();

   /**
    * mkdir creates a directory whose pathname is specified by this
    * FileResource.
    *
    * @return true if the directory could be created
    */
   boolean mkdir();

   /**
    * mkdirs creates a directory whose pathname is specified by this
    * FileResource, including any necessary parent directories.
    *
    * @return true if the directory (or directories) could be created
    */
   boolean mkdirs();

   /**
    * delete deletes the file specified by this object. If the target
    * file to be deleted is a directory, it must be empty for deletion to
    * succeed.
    *
    * @return true if the file is successfully deleted
    */
   boolean delete();
}

