package org.apache.xindice.server;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: User.java,v 1.1.1.1 2001/12/06 19:33:55 bradford Exp $
 */

import org.apache.xindice.util.*;
import java.io.*;
import java.util.*;

/**
 * User is the atomic representation of a Xindice user.
 * Users can be stored in an application-independent fashion using
 * custom implementations of UserManager. StdUserManager uses
 * Java's serialization mechanism to stream User instances to and from
 * a file in a directory on disk.
 *
 * TODO: At some point, this system will be replaced by a repository-
 * based user management system using XML as the storage and
 * representation format.
 *
 * @see org.apache.xindice.server.standard.StdUserManager
 */

public class User implements Serializable, Cloneable {
   static final long serialVersionUID = -6448739651072277147L;

   private static final String[] EmptyStrings = new String[0];
   
   protected transient boolean dirty = false;
   protected transient UserManager manager = null;

   protected String username = null;
   protected String password = null;
   protected boolean active = true;
   protected Map props = new HashMap();  // Serializable
   protected Map realms = new HashMap(); // Integer
   protected Map state = new HashMap();  // Serializable

   public User(String username, String password, Map props) {
      this.username = username;
      this.password = password;
      if ( props != null )
         this.props = props;
   }

   public User(String username, String password) {
      this(username, password, null);
   }

   public User(String username) {
      this(username, "");
   }

   /**
    * setUserManager attaches the originating UserManager to this User
    * in order to allow the User to save any changes that it has performed
    * back to the data store via the UserManager.
    *
    * @param manager The originating UserManager for this User
    */
   public void setUserManager(UserManager manager) {
      this.manager = manager;
   }

   /**
    * getUserManager returns the originating UserManager for this User.
    *
    * @return The originating UserManager for this User
    */
   public UserManager getUserManager() {
      return manager;
   }

   /**
    * setUsername sets the User's name, basically replacing the User in
    * the data store with a new User object.
    *
    * @param username The new username
    * @throws ParameterException if the username is taken or invalid
    */
   public void setUsername(String username) throws ParameterException {
      if ( manager != null && !username.equals(this.username) ) {
         if ( manager.getUser(username) != null )
            throw new ParameterException("User name", username, "User name "+username+" already exists");
         else {
            manager.deleteUser(this.username);
            this.username = username;
            save();
         }
      }
      else {
         this.username = username;
         dirty = true;
      }
   }

   /**
    * setPassword allows the User to replace their password.
    *
    * @param password The new password
    * @throws ParameterException if the password is invalid
    */
   public void setPassword(String password) throws ParameterException {
      this.password = password;
      dirty = true;
   }


   /**
    * setActive sets whether the User is actively available in the User
    * data store.  If the User is inactive, the authentication system
    * should not allow that User to log in or access any resources that
    * might be associated with them.
    *
    * @param active The new User active status
    */
   public void setActive(boolean active) {
      this.active = active;
      dirty = true;
   }

   /**
    * setRealm adds or replaces an authentication realm value in the
    * User's access control.
    *
    * @see org.apache.xindice.server.Script
    *
    * @param realm The realm to add
    * @param value The Access Methods the user will be afforded
    */
   public void setRealm(String realm, int value) {
      Integer ra = new Integer(value);
      realms.put(realm, ra);
      dirty = true;
   }

   /**
    * getRealm returns the authentication realm value that is associated
    * with the User's access control.
    *
    * @see org.apache.xindice.server.Script
    *
    * @param realm The realm to retrieve
    * @return The Access Methods the user can be afforded
    */
   public int getRealm(String realm) {
      Integer ra = (Integer)realms.get(realm);
      return ra != null ? ra.intValue()
                        : Script.ACCESS_NONE;
   }

   /**
    * setProperty sets a value in the User's property set.  User properties
    * are serialized and stored with the User in the data store.
    *
    * @param key The property name
    * @param value The property value
    * @return Whether or not the property was stored
    */
   public boolean setProperty(String key, Serializable value) {
      props.put(key, value);
      dirty = true;
      return true;
   }

   /**
    * getUsername returns the User's name.
    *
    * @return The User's name
    */
   public String getUsername() {
      return username;
   }

   /**
    * getPassword returns the User's password.
    *
    * @return The User's password
    */
   public String getPassword() {
      return password;
   }

   /**
    * getActive returns the User's actively available status.  If the User is
    * inactive, the authentication system should not allow that User to log in
    * or access any resources that might be associated with them.
    *
    * @return The User active status
    */
   public boolean getActive() {
      return active;
   }

   /**
    * getProperty returns the value for a property in the User's property
    * set.
    *
    * @param key The property name
    * @return The property value (or null)
    */
   public Object getProperty(String key) {
      return props.get(key);
   }

   /**
    * removeProperty removes a property from the User's property set.
    *
    * @param key The property name
    * @return Whether or not the property could be removed
    */
   public boolean removeProperty(String key) {
      props.remove(key);
      return true;
   }

   /**
    * appendProperty appends a value to an existing property in the User's
    * property set.  The specified delimiter will be used to delimit existing
    * values if there are any.
    *
    * @param key The property name
    * @param value The value to append
    * @param delimiter The delimiter to use
    * @return Whether or not the property value could be appended
    */
   public boolean appendProperty(String key, Serializable value, String delimiter) {
      String temp = (String)props.get(key);
      if ( temp != null && temp.length() > 0 )
         value = temp + delimiter + value.toString();
      props.put(key, value);
      dirty = true;
      return true;
   }

   /**
    * listProperties returns an array of Strings containing the names of all
    * of the properties that the User has available in their property set.
    *
    * @return The property names
    */
   public String[] listProperties() {
      return (String[])props.keySet().toArray(EmptyStrings);
   }

   /**
    * isDirty returns whether or not the User's record has been modified
    * since its last save.
    *
    * @return Whether or not the User's record has been modified
    */
   public boolean isDirty() {
      return dirty;
   }

   /**
    * save instructs the UserManager to save this User to its data store.
    *
    * @return Whether or not the User could be saved
    */
   public boolean save() {
      if ( dirty && manager != null ) {
         manager.setUser(this);
         dirty = false;
         return true;
      }
      return false;
   }

   /* Persistent state methods */

   /**
    * setPersistentState allows a Script to store the Gateway's current
    * session state to the current User and serialize that state as
    * part of the User's image in the UserManager's data store.  Because
    * Session State can exist without a User, and vice versa, this method
    * is a convenient tool for storing a User's personalized information.
    *
    * @param gateway The Gateway to query for session state
    */
   public void setPersistentState(Gateway gateway) {
      Iterator enum = gateway.enumObjects();
      String key;
      Object obj;
      while ( enum.hasNext() ) {
         key = (String)enum.next();
         obj = gateway.getObject(key);
         if ( (obj instanceof Serializable) && !(obj instanceof Transient) )
            state.put(key, obj);
      }
      dirty = true;
   }

   /**
    * getPersistentState allows a Script to retrieve the Gateway's current
    * session state from the current User, having been serialized as
    * part of the User's image in the UserManager's data store.  Because
    * Session State can exist without a User, and vice versa, this method
    * is a convenient tool for storing a User's personalized information.
    *
    * @param gateway The Gateway to store the session state in
    */
   public void getPersistentState(Gateway gateway) {
      Iterator enum = state.keySet().iterator();
      String key;
      Object obj;
      while ( enum.hasNext() ) {
         key = (String)enum.next();
         obj = state.get(key);
         gateway.setObject(key, obj);
      }
   }

   /**
    * removePersistentState allows a Script to completely remove all
    * serialized Session state from the User's record.
    */
   public void removePersistentState() {
      state.clear();
   }

   protected void finalize() throws Throwable {
      save();
   }

   /**
    * Serializable clone() implementation - The easy way to do it!
    *
    * @return The cloned Object
    * @exception CloneNotSupportedException If an error occurs.
    */
   public Object clone() throws CloneNotSupportedException {
      try {
         ByteBuffer buf = new ByteBuffer(1024);
         new ObjectOutputStream(buf).writeObject(this);
         User clone = (User)new ObjectInputStream(buf.getInputStream()).readObject();
         clone.manager = manager;
         clone.dirty = dirty;
         return clone;
      }
      catch ( Exception e ) {
         // For some reason, we couldn't serialize the object
         // therefore, we can't clone... Throw an exception.
         throw new CloneNotSupportedException();
      }
   }
}

