package org.apache.xindice.server.services;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: HTTPServer.java,v 1.3 2002/03/11 22:32:36 kstaken Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * HTTPServer is as close to an HTTP/1.1 implementation as you can get
 * in a script-based web server. Except for localization functionality, nearly
 * every element of the HTTP/1.1 specification is supported.
 * <pre>
 * TODO:
 *    Optimization - translatePath could use a good beating
 *    Optimization - Header Parsing could be sped up by removing generic
 *                   routines and using inline code instead.
 * </pre>
 */

public final class HTTPServer extends SocketServer {
   private static final String GatewayType = "CGI/1.0";
   private static final String HexChars = "0123456789ABCDEF";
   
   private static final String NAME = "name";
   private static final String DOCROOT = "docroot";
   private static final String RESPONSES = "responses";
   private static final String RESPONSE = "response";
   private static final String CODE = "code";
   private static final String MESSAGE = "message";
   private static final String METHODS = "methods";
   private static final String METHOD = "method";
   private static final String COMMAND = "command";
   private static final String SCRIPT = "script";
   private static final String ALIASES = "aliases";
   private static final String ALIAS = "alias";
   private static final String PATH = "path";
   private static final String VSERVERS = "vservers";
   private static final String VSERVER = "vserver";
   private static final String HOST = "host";
   
   protected Map msgs = new HashMap();    // String
   protected Map methods = new HashMap(); // String
   protected Map aliases = new HashMap(); // String
   protected Map docroots = new HashMap(); // String
   
   protected String redirect = null;
   protected String docroot = null;
   
   private ObjectPool gateways = new ObjectPool() {
      protected Poolable createObject() {
         return new HTTPGateway();
      }
   };
   
   public HTTPServer() {
      port = 80;
   }
   
   public void setConfig(Configuration config) throws XindiceException {
      super.setConfig(config);

      docroot = config.getAttribute(DOCROOT, "./docs/");
      if (! docroot.startsWith("/")) {
          docroot = System.getProperty( Xindice.PROP_XINDICE_HOME ) +
             "/" + docroot;
      }
          
      Configuration vserverConfig = config.getChild(VSERVERS);
      if ( vserverConfig != null ) {
         vserverConfig.processChildren(VSERVER,
            new ConfigurationCallback() {
               public void process(Configuration cfg) {
                  docroots.put(cfg.getAttribute(HOST), cfg.getAttribute(DOCROOT));
               }
         });
      }
      
      Configuration responseConfig = config.getChild(RESPONSES);
      if ( responseConfig != null ) {
         responseConfig.processChildren(RESPONSE,
            new ConfigurationCallback() {
               public void process(Configuration cfg) {
                  msgs.put(cfg.getAttribute(CODE), cfg.getAttribute(MESSAGE));
               }
         });
      }
      
      Configuration methodsConfig = config.getChild(METHODS);
      if ( methodsConfig != null ) {
         methodsConfig.processChildren(METHOD,
            new ConfigurationCallback() {
               public void process(Configuration cfg) {
                  methods.put(cfg.getAttribute(COMMAND), cfg.getAttribute(SCRIPT));
               }
         });
      }
      
      Configuration aliasConfig = config.getChild(ALIASES);
      if ( aliasConfig != null ) {
         aliasConfig.processChildren(ALIAS,
            new ConfigurationCallback() {
               public void process(Configuration cfg) {
                  aliases.put(cfg.getAttribute(NAME), cfg.getAttribute(PATH));
               }
         });
      }
   }
   
   public String getProtocol() {
      return "http";
   }
   
   public synchronized int initialize() {
      serverGroup = new ThreadGroup(getName());
      connectionGroup = new ThreadGroup(serverGroup, "HTTP Gateways");
      return RESULT_OK;
   }
   
   public void run() {
      if ( !createServerSocket() )
         return;
      
      redirect = "http://"+hostname;
      if ( port != 80 )
         redirect += ":"+port;
      
      try {
         HTTPGateway gateway;
         while ( true ) {
            gateway = (HTTPGateway)gateways.getObject();
            gateway.runWith(sock.accept());
         }
      }
      catch ( Exception e ) {
      }
   }
   
   /**
    * HTTPGateway
    */
   
   private class HTTPGateway extends Gateway implements Runnable, Poolable {
      private Thread connection;
      private Socket sock;
      private byte[] postdata;
      private FileResource postfile;
      private boolean keepalive = true;
      private boolean connected = true;
      private String aliasname;
      private String scriptbase;
      private String scriptpath;
      private String scriptname;
      private String version;
      private String hostaddress;
      private String hostname;
      private String pathinfo;
      private int lastcode = 200;
      private String message;
      
      private ByteBuffer content = new ByteBuffer(2048);
      
      /* Variables Copied From HTTPServer */
      
      private ObjectPool pool;
      private Properties props;
      private int keepcount;
      
      protected HTTPGateway() {
         setKernel(HTTPServer.this.kernel);
         
         connection = new Thread(connectionGroup, this, "HTTP Gateway");
         connection.setDaemon(true);
      }
      
      public void setPool(ObjectPool pool) {
         this.pool = pool;
      }
      
      public void reclaim() {
         if ( pool != null )
            pool.putObject(this);
      }
      
      protected void runWith(Socket sock) {
         this.sock = sock;
         if ( connection.isAlive() ) {
            synchronized(this) {
               notify();
            }
         }
         else
            connection.start();
      }
      
      public void run() {
         int hits = 0;
         
         Gateway.setCurrentGateway(this);
         while ( true ) {
            try {
               sock.setTcpNoDelay(true);
               sock.setSoLinger(true, linger);
               sock.setSoTimeout(timeout);
            }
            catch ( Exception e ) {
            }
            
            message = (String)msgs.get("200");
            
            hostaddress = sock.getInetAddress().getHostAddress();
            hostname = reverse ? sock.getInetAddress().getHostName()
               : hostaddress;
            do {
               hits++;
               init();
               if ( status() == Service.STATE_STOPPED )
                  break;
               if ( hits == keepcount )
                  setKeepAlive(false);
               if ( connected ) {
                  if ( translatePath() )
                     this.kernel.runScript(getScriptName(), this, true);
                  else {
                     logMessage(LogManager.LOG_ERROR, "Bad Request: "+pathinfo);
                     sendError(400, "Bad Request");
                  }
               }
            }
            while ( connected && keepalive );
            try {
               sock.close();
            }
            catch ( Exception e ) {
               System.err.println("\u0007ERROR: Closing HTTP client socket");
            }
            
            // Perform Some Cleanup
            keepalive = true;
            hits = 0;
            
            // Return The Gateway To The Connection Pool
            reclaim();
            synchronized(this) {
               try {
                  wait();
               }
               catch ( Exception e ) {
               }
            }
         }
      }
      
      protected void init() {
         super.init();
         content.reset();
         postdata = null;
         postfile = null;
         connected = true;
         
         // Set default header values
         resHdr.put("Date", DateUtilities.getGMTString(new Date()));
         resHdr.put("Server", "Xindice."+Xindice.Version);
         resHdr.put("Cache-control", "no-cache");
         resHdr.put("Content-type", "text/html");
         
         // Parse the header out
         try {
            DataInputStream dis = new DataInputStream(new BufferedInputStream(sock.getInputStream(), 2048));
            String rline;  // Line read from Socket Input Stream
            String rm;     // Parsed request method (GET, PUT, etc...)
            String query;  // Parsed query string (?QUERY=VALUE& etc...)
            
            // Get the first line of the request
            try {
               rline = dis.readLine().replace('\t', ' ').trim();
            }
            catch ( Exception e ) {
               connected = false;
               return;
            }
            
            int idx = rline.indexOf(' ');
            if ( idx != -1 ) {
               rm = rline.substring(0, idx).trim();
               rline = rline.substring(idx+1);
            }
            else {
               rm = rline;
               rline = "";
            }
            
            idx = rline.lastIndexOf(' ');
            if ( idx != -1 ) {
               version = rline.substring(idx+1).trim().toUpperCase();
               if ( version.startsWith("HTTP/") )
                  rline = rline.substring(0, idx);
               else
                  version = "";
            }
            else
               version = "";
            
            idx = rline.indexOf('(');
            if ( idx != -1 ) {
               query = java.net.URLDecoder.decode(rline.substring(idx).trim());
               rline = rline.substring(0, idx);
            }
            else {
               idx = rline.indexOf('?');
               if ( idx != -1 ) {
                  query = rline.substring(idx+1).trim();
                  rline = rline.substring(0, idx);
               }
               else
                  query = "";
            }
            
            pathinfo = java.net.URLDecoder.decode(rline.trim());
            
            reqHdr.put("REQUEST_METHOD", rm);
            reqHdr.put("PATH_INFO", pathinfo);
            reqHdr.put("QUERY_STRING", query);
            
            reqHdr.put("REMOTE_HOST", hostname);
            reqHdr.put("REMOTE_ADDR", hostaddress);
            
            // Get the header values, parse them and put them into the Client table
            String line;
            String key;
            String value;
            do {
               try {
                  line = dis.readLine();
               }
               catch ( Exception e ) {
                  connected = false;
                  return;
               }
               if ( line == null )
                  line = "";
               idx = line.indexOf(':');
               if ( idx != -1 ) {
                  key = line.substring(0, idx).trim();
                  value = line.substring(idx+1).trim();
                  key = key.replace('_', '-');
                  setRequestHeader(key, value);
               }
            }
            while ( line.length() > 0 );
            
            if ( version.length() == 0 || version.equals("HTTP/1.0") )
               keepalive = getRequestHeader("Connection").toUpperCase().equalsIgnoreCase("keep-alive");
            else
               keepalive = !getRequestHeader("Connection").toUpperCase().equalsIgnoreCase("close");
            
            setKeepAlive(keepalive);
            
            // Parse out the form variables from the QUERY_STRING variable
            if ( !query.startsWith("(") )
               parseBuffer(values, query, "&?;", true, true);
            // Parse out the cookies from the COOKIE variable
            parseBuffer(cookies, getRequestHeader("Cookie"), ";", false, true);
            
            // Everything after this point will be post data if there is any
            String cl = getRequestHeader("Content-length");
            if ( cl.length() > 0 ) {
               int postlen = StringUtilities.stringToInt(cl, 0);
               if ( postlen > 0 ) {
                  boolean form = getRequestHeader("Content-type").equalsIgnoreCase("application/x-www-form-urlencoded");
                  if ( form || postlen <= chunkSize ) {
                     // Store the post data in memory
                     postdata = new byte[postlen];
                     try {
                        dis.read(postdata);
                     }
                     catch ( Exception e ) {
                        connected = false;
                        return;
                     }
                     if ( form )
                        parseBuffer(values, new String(postdata), "&?;", true, true);
                  }
                  else {
                     // Store the post data in a temporary file
                     postfile = this.kernel.getTemporaryFileResource();
                     try {
                        int count = 0;
                        byte[] buffer = new byte[chunkSize];
                        OutputStream os = postfile.getOutputStream();
                        do {
                           count = dis.read(buffer);
                           if ( count > 0 )
                              os.write(buffer, 0, count);
                        }
                        while ( count == chunkSize );
                        os.close();
                     }
                     catch ( Exception e ) {
                        connected = false;
                        return;
                     }
                  }
               }
            }
         }
         catch ( Exception e ) {
            // An Exception Here Most Likely Means The Connection Was Closed
            keepalive = false;
            connected = false;
         }
      }
      
      public String getGatewayType() {
         return GatewayType;
      }
      
      public OutputStream getOutputStream() {
         return content;
      }
      
      public void resetOutputStream() {
         content.reset();
      }
      
      public OutputStream getNativeOutputStream() {
         try {
            return sock.getOutputStream();
         }
         catch ( Exception e ) {
            System.err.println("\u0007ERROR: "+e);
         }
         return null;
      }
      
      public InputStream getInputStream() {
         if ( postfile != null )
            return postfile.getInputStream();
         else
            return new ByteArrayInputStream(postdata);
      }
      
      public ByteBuffer getContent() {
         return content;
      }
      
      public ByteBuffer getHeaderContent(int code) {
         ByteBuffer buf = new ByteBuffer(1024);
         try {
            if ( version.length() != 0 ) {
               buf.append(version);
               buf.append(' ');
               buf.append(code);
               if ( code != lastcode ) {
                  message = (String)msgs.get(""+code);
                  lastcode = code;
               }
               if ( message != null ) {
                  buf.append(' ');
                  buf.append(message);
               }
               buf.append("\r\n");
            }
            Iterator enum = resHdr.keySet().iterator();
            String key;
            while ( enum.hasNext() ) {
               key = (String)enum.next();
               if ( key.charAt(0) == '@' ) {
                  buf.append("Set-Cookie: ");
                  buf.append(key.substring(1));
                  buf.append('=');
               }
               else {
                  buf.append(key);
                  buf.append(": ");
               }
               buf.append(resHdr.get(key));
               buf.append("\r\n");
            }
            buf.append("\r\n");
         }
         catch ( Exception e ) {
            // No Exception Should Occur Here
         }
         return buf;
      }
      
      public void send(int code) {
         if ( sock != null ) {
            try {
               resHdr.put("Content-Length", ""+content.size());
               OutputStream os = sock.getOutputStream();
               getHeaderContent(code).writeTo(os);
               if ( !getRequestHeader("REQUEST_METHOD").equals("HEAD") )
                  content.chunkTo(os, chunkSize);
            }
            catch ( Exception e ) {
               keepalive = false;
               connected = false;
            }
         }
      }
      
      public void send() {
         send(200);
      }
      
      private void addFooter(PrintStream out) {
         out.println("<HR ALIGN=\"CENTER\" SIZE=\"1\" NOSHADE>");
         out.println("<P><FONT FACE=\"Arial, Helvetica\"><B>"+Xindice.Title+"</B> "+Xindice.Version+" ("+Xindice.Codename+")<BR />");
         out.println("&copy; "+Xindice.Year+" <A HREF=\""+Xindice.URL+"\" TARGET=\"_new\">"+Xindice.Company+"</A><BR />");
         out.println("All Rights Reserved</FONT></P>");
      }
      
      public void sendError(int code, String message) {
         resetOutputStream();
         resHdr.put("Content-type", "text/html");
         PrintStream out = new PrintStream(getOutputStream());
         out.println("<HTML><HEAD><TITLE>Xindice.Error</TITLE></HEAD><BODY BGCOLOR=\"#FFFFFF\">");
         out.println("<P><FONT FACE=\"Helvetica,Helv,Arial\">"+message+"<BR /></FONT></P>");
         addFooter(out);
         out.println("</BODY></HTML>");
         send(code);
      }
      
      public void sendError(int code) {
         sendError(code, "");
      }
      
      public void sendError() {
         sendError(500);
      }
      
      public void sendException(String name, Exception e) {
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         BufferedOutputStream buf = new BufferedOutputStream(bos, 256);
         PrintStream ps = new PrintStream(buf);
         e.printStackTrace(ps);
         ps.flush();
         StringBuffer sb = new StringBuffer();
         sb.append("Error Executing "+name+"<PRE>");
         StringTokenizer st = new StringTokenizer(new String(bos.toByteArray()), "\n\r");
         while ( st.hasMoreTokens() )
            sb.append("<BR />"+st.nextToken());
         sb.append("</PRE>");
         sendError(500, sb.toString());
      }
      
      public void redirectLocal(String URI) {
         String host = getRequestHeader("Host");
         if ( host.length() > 0 )
            redirectRemote(combinePaths("http://"+host, URI));
         else
            redirectRemote(combinePaths(getRootURL(), URI));
      }
      
      public void redirectRemote(String URL) {
         resetOutputStream();
         resHdr.put("Content-type", "text/html");
         PrintStream out = new PrintStream(getOutputStream());
         out.println("<HTML><HEAD><TITLE>Document Moved</TITLE>");
         out.println("<META HTTP-EQUIV=\"Refresh\" CONTENT=\"0;URL="+URL+"\"></HEAD><BODY BGCOLOR=\"#FFFFFF\" />");
         out.println("This document has been moved to<BR />");
         out.println("<A HREF=\""+URL+"\">"+URL+"</A>");
         addFooter(out);
         out.println("</BODY></HTML>");
         resHdr.put("Location", URL);
         send(302);
      }
      
      /* Local Inline Implementations Of Some Of StringUtilities' Methods */
      
      private String combinePaths(String base, String relative) {
         int bl = base.length()-1;
         int rl = relative.length()-1;
         StringBuffer sb = new StringBuffer(bl+rl+3);
         sb.append(base);
         if ( bl >= 0 && base.charAt(bl) != '/' )
            sb.append('/');
         sb.append(rl >= 0 && relative.charAt(0) == '/' ? relative.substring(1)
                      : relative);
         return sb.toString();
      }
      
      private void parseBuffer(Map retprops, String inbuffer, String delimiters, boolean urldecode, boolean multivalue) {
         if ( inbuffer.length() == 0 )
            return;
         int idx = 0;
         String line;
         String key;
         String value;
         String prop;
         String temp;
         StringTokenizer st = new StringTokenizer(inbuffer, delimiters, false);
         while ( st.hasMoreTokens() ) {
            line = st.nextToken();
            idx = line.indexOf('=');
            if ( idx != -1 ) {
               key = line.substring(0, idx).trim();
               try {
                  value = urldecode ? URLDecoder.decode(line.substring(idx+1).trim())
                     : line.substring(idx+1).trim();
               }
               catch ( Exception e ) {
                  value = line.substring(idx+1).trim();
               }
               if ( multivalue ) {
                  temp = (String)retprops.get(key);
                  if ( temp != null && temp.length() > 0 )
                     value = temp + "\u0001" + value;
                  retprops.put(key, value);
               }
               else
                  retprops.put(key, value);
            }
         }
      }
      
      /* Script Path Information */
      
      private boolean translatePath() {
         // This method could use a lot of optimization at some point
         aliasname = "";
         scriptname = "";
         
         StringTokenizer st;
         String token;
         int idx = -1;
         
         // Translate the pathinfo variable, expanding aliases to their correct
         // absolute paths.  Leave the PATH_INFO client variable alone.
         
         String hostName = getRequestHeader("HOST");
         String root = (String)docroots.get(hostName);
         if ( root == null )
            root = docroot;
         
         // Translate the document root if necessary
         if ( pathinfo.equals("/") || pathinfo.length() == 0 )
            pathinfo = combinePaths(root, "");
         else {
            // Translate the alias path if there is one
            idx = pathinfo.indexOf('/', 1);
            token = idx != -1 ? pathinfo.substring(1, idx)
               : pathinfo.substring(1);
            String newpath = (String)aliases.get(token);
            if ( newpath != null && newpath.length() > 0 ) {
               aliasname = token;
               pathinfo = combinePaths(newpath, pathinfo.substring(token.length()+1));
            }
            else
               pathinfo = combinePaths(root, pathinfo);
            // Remove trailing dots from the URI if there are any
            idx = pathinfo.length()-1;
            if ( pathinfo.charAt(idx) == '.' ) {
               while ( pathinfo.charAt(--idx) == '.' );
               pathinfo = pathinfo.substring(0, idx+1);
            }
         }
         
         // Check for a specific method for this extension.  If we can't find one,
         // Use the more generally defined method in its place.  Finally, check to
         // see if the method found is an enforced method.  If it is, we'll use that
         // instead of the script defined in the URL.
         
         StringBuffer base = new StringBuffer(pathinfo.length());
         StringBuffer path = new StringBuffer(pathinfo.length());
         String requestmethod = getRequestHeader("REQUEST_METHOD");
         String method = "";
         
         idx = pathinfo.lastIndexOf('.');
         if ( idx != -1 )
            scriptname = (String)methods.get(requestmethod+pathinfo.substring(idx));
         if ( scriptname == null )
            scriptname = (String)methods.get(requestmethod);
         
         // Parse through the path information for the ScriptBase, ScriptPath,
         // and ScriptName (if one hasn't already been enforced).
         
         boolean foundscript = false;
         st = new StringTokenizer(pathinfo, "/", true);

         // protect against platform-specific file separator attacks
         String tokens = "/";
         if ( ! File.separator.equals(tokens) )
            tokens += File.separator;
         
         st = new StringTokenizer(pathinfo, tokens, true);
         while ( st.hasMoreTokens() ) {
            token = st.nextToken();
            if ( !foundscript && token.endsWith(".jug") ) {
               scriptname = token.substring(0, token.length()-4);
               foundscript = true;
            }
            else if ( token.equals("..") )
               return false;
            else if ( foundscript )
               path.append(token);
            else
               base.append(token);
         }
         if ( foundscript ) {
            scriptbase = base.toString();
            scriptpath = path.toString();
         }
         else {
            scriptbase = "";
            scriptpath = pathinfo;
         }
         
         if ( scriptname == null )
            scriptname = "";
         reqHdr.put("SCRIPT_BASE", scriptbase);
         reqHdr.put("SCRIPT_PATH", scriptpath);
         reqHdr.put("SCRIPT_NAME", scriptname);
         reqHdr.put("PATH_TRANSLATED", pathinfo);
         return true;
      }
      
      public String getRootURL() {
         return redirect;
      }
      
      public String getScriptName() {
         return scriptname;
      }
      
      public String getScriptPath() {
         return scriptpath;
      }
      
      public String getScriptBase() {
         return scriptbase;
      }
      
      public String getPathAlias() {
         return aliasname;
      }
      
      private void setKeepAlive(boolean value) {
         keepalive = value;
         resHdr.put("Connection", keepalive ? "Keep-Alive"
                       : "Close");
      }
   }
}



