package org.apache.xindice.server.services;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: SocketServer.java,v 1.1.1.1 2001/12/06 19:33:56 bradford Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import java.net.*;
import java.util.*;

/**
 * SocketServer is the generic base class for all of Xindice's
 * Socket-based Services.
 */

public abstract class SocketServer extends SimpleConfigurable implements Service, KernelAccess, Runnable {
   private static final String BIND = "bind";
   private static final String HOST = "host";
   private static final String PORT = "port";
   private static final String KEEPALIVE = "keepalive";
   private static final String LINGER = "linger";
   private static final String TIMEOUT = "timeout";
   private static final String CHUNKSIZE = "chunksize";
   private static final String REVERSE = "reverse";
   
   private static final String NAME = "name";
   private static String[] StateStrings = {"Stopped", "Started", "Suspended"};

   private int status = STATE_STOPPED;
   private String statusmsg = "Not Initialized";
   private String holdmsg = null;
      
   protected ThreadGroup serverGroup = null;
   protected ThreadGroup connectionGroup = null;
   protected Thread server = null;
   protected ServerSocket sock = null;

   // TODO: These variables shouldn't really be public
   public Kernel kernel = null;
   
   public int port = 0;
   public String hostname = null;
   public int keepalive = 16;
   public int linger = 1000;
   public int timeout = 300000;
   public boolean reverse = true;
   public int chunkSize = 32768;
   
   
   public void setKernel(Kernel kernel) {
      this.kernel = kernel;
   }

   public void setConfig(Configuration config) throws XindiceException {
      super.setConfig(config);
      
      // These attributes are stored in a "bind" element so
      // they don't conflict with anything in the "service"
      // element.
      Configuration bindConfig = config.getChild(BIND);
      hostname = bindConfig.getAttribute(HOST);
      port = bindConfig.getIntAttribute(PORT);
      keepalive = bindConfig.getIntAttribute(KEEPALIVE);
      linger = bindConfig.getIntAttribute(LINGER);
      timeout = bindConfig.getIntAttribute(TIMEOUT)*1000;
      chunkSize = bindConfig.getIntAttribute(CHUNKSIZE);
      reverse = bindConfig.getBooleanAttribute(REVERSE);
   }
   
   public String getName() {
      return config.getAttribute(NAME);
   }

   public int status() {
      return status;
   }

   public String statusMessage() {
      return statusmsg;
   }

   public abstract String getProtocol();
   
   public String getHostname() {
      return hostname;
   }
   
   public int getPort() {
      return port;
   }
   
   public synchronized int start() {
      if ( status == STATE_STOPPED ) {
         server = new Thread(serverGroup, this, getName());
         statusmsg = "Starting";
         server.start();
         try {
            wait();
         }
         catch ( Exception e ) {
         }
         return status == STATE_STARTED ? RESULT_OK
                                        : RESULT_FAILED;
      }
      else
         return status == STATE_SUSPENDED ? resume()
                                          : RESULT_FAILED;
   }

   public synchronized int stop() {
      if ( status == STATE_STARTED || status == STATE_SUSPENDED ) {
         server.interrupt();
         try {
            sock.close();
         }
         catch ( Exception e ) {
            System.err.println("\u0007ERROR: Closing '"+getName()+"' server socket");
         }
         server.stop();
         //while ( server.isAlive() )
         //   Thread.yield();
         server = null;
         status = STATE_STOPPED;
         statusmsg = "Not listening";
         return RESULT_OK;
      }
      else
         return RESULT_FAILED;
   }

   public synchronized int suspend() {
      if ( status == STATE_STARTED ) {
         server.suspend();
         status = STATE_SUSPENDED;
         holdmsg = statusmsg;
         statusmsg = "Listening suspended";
         return RESULT_OK;
      }
      else
         return RESULT_FAILED;
   }

   public synchronized int resume() {
      if ( status == STATE_SUSPENDED ) {
         server.resume();
         status = STATE_STARTED;
         statusmsg = holdmsg;
         return RESULT_OK;
      }
      else
         return status == STATE_STOPPED ? start()
                                        : RESULT_FAILED;
   }

   public synchronized int uninitialize() {
      if ( status() == STATE_STARTED )
         stop();
      setStatusMessage("Not initialized");
      server = null;
      return RESULT_OK;
   }

   public abstract int initialize();
   public abstract void run();

   /**
    * createServerSocket creates the listening socket for the SocketServer.
    *
    * @return Whether or not the socket is listening
    */
   protected boolean createServerSocket() {
      notify: {
         InetAddress bindaddr = null;
         try {
            if ( hostname.length() > 0 ) {
               bindaddr = InetAddress.getByName(hostname);
               sock = new ServerSocket(port, 5, bindaddr);
            }
            else {
               bindaddr = InetAddress.getLocalHost();
               hostname = bindaddr.getHostName();
               sock = new ServerSocket(port);
            }
         }
         catch ( Exception ie ) {
            setStatus(STATE_STOPPED, "Could not listen at port "+port+" on "+bindaddr.getHostName());
            break notify;
         }
         setStatus(STATE_STARTED, "Listening at port "+port+" on "+bindaddr.getHostName());
      }
      setNotified();
      
      return status == STATE_STARTED;
   }
      
   /**
    * Allows a Service to set the reported status() and
    * statusMessage() values.
    *
    * @param status The status value
    * @param statusmsg The status message value
    */
   protected void setStatus(int status, String statusmsg) {
      this.status = status;
      this.statusmsg = statusmsg;
   }

   /**
    * Allows a Service to set the reported statusMessage()
    * value.
    *
    * @param statusmsg The status message value
    */
   protected void setStatusMessage(String statusmsg) {
      this.statusmsg = statusmsg;
   }

   /**
    * Allows a Service to set the reported status() value.  A
    * generic statusMessage() value will be generated based on the
    * status value provided.
    *
    * @param status The status value
    */
   protected void setStatus(int status) {
      try {
         setStatus(status, getName()+" "+StateStrings[status]);
      }
      catch ( Exception e ) {
         setStatus(status, getName()+" Status Unknown");
      }
   }

   /**
    * Notifies the ServiceManager interface that the Service
    * is ready for requests.  This method must be called by Services
    * that extend SocketServer.
    */
   protected synchronized void setNotified() {
      notify();
   }
}


