package org.apache.xindice.server.standard;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: StdComponentManager.java,v 1.1.1.1 2001/12/06 19:33:56 bradford Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import java.io.*;
import java.util.*;

/**
 * StdComponentManager
 */

public final class StdComponentManager implements Configurable, ComponentManager, KernelAccess, Disposable {
   private static final String COMPONENT = "component";
   private static final String CLASS = "class";
   
   private Kernel kernel;
   private Configuration config;
   private Map components = Collections.synchronizedMap(new HashMap()); // of ScriptComponent
   private List filters = Collections.synchronizedList(new ArrayList()); // of ScriptFilter

   public void setKernel(Kernel kernel) {
      this.kernel = kernel;
   }
   
   public void setConfig(Configuration config) {
      this.config = config;
      config.processChildren(COMPONENT,
         new ConfigurationCallback() {
            public void process(Configuration cfg) {
               String className = cfg.getAttribute(CLASS);
               try {
                  Object obj = Class.forName(className).newInstance();
                  if ( obj instanceof ScriptComponent ) {
                     ScriptComponent sc = (ScriptComponent)obj;
                     kernel.setKernelOf(sc);
                     if ( obj instanceof Configurable )
                        ((Configurable)obj).setConfig(cfg);
                     if ( !addComponent(sc) )
                        System.err.println("\u0007ERROR: Class '"+className+"' could not be added as a component");
                  }
                  else
                     System.err.println("\u0007ERROR: Class '"+className+"' is not a ScriptComponent");
               }
               catch ( ClassNotFoundException e) {
                  System.err.println("\u0007ERROR: Component class '"+className+"' not found");
               }
               catch ( Exception e) {
                  System.err.println("\u0007ERROR: initializing '"+className+"'");
                  e.printStackTrace(System.err);
               }
            }
      });
   }

   public Configuration getConfig() {
      return config;
   }
   
   public boolean addComponent(ScriptComponent component, String name) {
      Object obj = components.get(name);
      if ( obj == null ) {
         components.put(name, component);
         Class cclass = component.getClass();
         boolean added = false;
         if ( component instanceof ScriptFilter ) {
            System.out.println("Filter: '"+name+"' added to filtering chain");
            filters.add(component);
            added = true;
         }
         if ( component instanceof Script ) {
            System.out.println("Script: '"+name+"' added to script storage");
            kernel.addScript(name, (Script)component);
            added = true;
         }
         if ( !added )
            System.out.println("Component: '"+name+"' added to component storage");
         return true;
      }
      else
         return false;
   }

   public boolean addComponent(ScriptComponent component) {
      return addComponent(component, component.getName());
   }

   public ScriptComponent getComponent(String name) {
      return (ScriptComponent)components.get(name);
   }

   public ScriptFilter[] getFilters(Versioning versioning) {
      List l = new ArrayList();
      ScriptFilter filter;
      int i = 0;
      int size = filters.size();
      for ( i = 0; i < size; i++ ) {
         filter = (ScriptFilter)filters.get(i);
         if ( filter.isPackageFiltered(versioning.getPackageName()) )
            l.add(filter);
      }
      size = l.size();
      ScriptFilter[] result = new ScriptFilter[size];
      for ( i = 0; i < size; i++ )
         result[i] = (ScriptFilter)l.get(i);
      return result;
   }

   public boolean removeComponent(String name) {
      Object obj = components.remove(name);
      filters.remove(obj);
      Kernel.disposeOf(obj);
      return true;
   }

   public void dispose() {
      Iterator enum = components.values().iterator();
      while ( enum.hasNext() )
         Kernel.disposeOf(enum.next());
   }
}

