package org.apache.xindice.util;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: ByteBuffer.java,v 1.1.1.1 2001/12/06 19:33:57 bradford Exp $
 */

import java.io.*;

/**
 * ByteBuffer manages volatile arrays of bytes. It is implemented with
 * nearly the same method prototypes as StringBuffer except for the
 * setByteAt and byteAt methods which replace
 * StringBuffer's setCharAt and charAt methods.
 * ByteBuffer also extends the OutputStream abstract class and
 * functions like the ByteArrayOutputStream class.
 * getInputStream returns an InputStream instance for
 * streaming in the content of the ByteBuffer. Because none of the
 * methods in ByteBuffer are synchronized, it is the user's job to
 * properly synchronize calls to any of the methods of this class.
 */

public final class ByteBuffer extends OutputStream implements Serializable {
   static final long serialVersionUID = -3900903004630456844L;

   private transient int length = 0;

   private byte[] buffer = null;
   private boolean locked = false;

   public ByteBuffer(int size) {
      buffer = new byte[size];
   }

   public ByteBuffer(String value) {
      length = value.length();
      buffer = new byte[length];
      char[] temp = value.toCharArray();
      length = temp.length;
      for ( int i = 0; i < length; i++ )
         buffer[i] = (byte)temp[i];
   }

   public ByteBuffer(StringBuffer value) {
      this(value.toString());
   }

   public ByteBuffer(byte[] value, boolean copy) {
      length = value.length;
      if ( copy ) {
         buffer = new byte[length];
         System.arraycopy(value, 0, buffer, 0, length);
      }
      else
         buffer = value;
   }

   public ByteBuffer(byte[] value) {
      length = value.length;
      buffer = new byte[length];
      System.arraycopy(value, 0, buffer, 0, length);
   }

   public ByteBuffer() {
      this(64);
   }

   public int capacity() {
      return buffer.length;
   }

   public void ensureCapacity(int size) {
      if ( locked )
         return;
      checkCapacity(size);
   }

   private void checkCapacity(int size) {
      if ( size > buffer.length ) {
         int newsize = (buffer.length+1)*2;
         if ( size > newsize )
            newsize = size;
         byte[] old = buffer;
         buffer = new byte[newsize];
         System.arraycopy(old, 0, buffer, 0, old.length);
      }
   }

   public void pack() {
      if ( locked )
         return;
      if ( buffer.length > length ) {
         byte[] value = new byte[length];
         System.arraycopy(buffer, 0, value, 0, length);
         buffer = value;
      }
   }

   /* StringBuffer style methods */

   public void append(byte[] value) {
      if ( locked )
         return;
      checkCapacity(length+value.length);
      System.arraycopy(value, 0, buffer, length, value.length);
      length += value.length;
   }

   public void append(char[] value) {
      if ( locked )
         return;
      checkCapacity(length+value.length);
      int len = value.length;
      for ( int i = 0; i < len; i++ )
         buffer[length+i] = (byte)value[i];
      length += value.length;
   }

   public void append(String value) {
      append(value.toCharArray());
   }

   public void append(StringBuffer value) {
      append(value.toString().toCharArray());
   }

   public void append(ByteBuffer value) {
      try {
         value.writeTo(this);
      }
      catch ( Exception e ) {
      }
   }

   public void append(boolean value) {
      append(String.valueOf(value));
   }

   public void append(char value) {
      if ( locked )
         return;
      checkCapacity(length+1);
      buffer[length++] = (byte)value;
   }

   public void append(byte value) {
      if ( locked )
         return;
      checkCapacity(length+1);
      buffer[length++] = value;
   }

   public void append(double value) {
      append(String.valueOf(value));
   }

   public void append(float value) {
      append(String.valueOf(value));
   }

   public void append(int value) {
      append(String.valueOf(value));
   }

   public void append(long value) {
      append(String.valueOf(value));
   }

   public void append(Object value) {
      append(String.valueOf(value));
   }

   public void insert(byte[] value, int pos) {
      if ( locked )
         return;
      if ( pos < 0 || pos >= length )
         throw new IndexOutOfBoundsException();
      checkCapacity(length+value.length);
      byte[] temp = new byte[length-pos];
      System.arraycopy(buffer, pos, temp, 0, temp.length);
      System.arraycopy(temp, 0, buffer, pos+value.length, temp.length);
      System.arraycopy(value, 0, buffer, pos, value.length);
      length += value.length;
   }

   public void insert(char[] value, int pos) {
      if ( locked )
         return;
      if ( pos < 0 || pos >= length )
         throw new IndexOutOfBoundsException();
      checkCapacity(length+value.length);
      byte[] temp = new byte[length-pos];
      System.arraycopy(buffer, pos, temp, 0, temp.length);
      System.arraycopy(temp, 0, buffer, pos+value.length, temp.length);
      int len = value.length;
      for ( int i = 0; i < len; i++ )
         buffer[pos+i] = (byte)value[i];
      length += value.length;
   }

   public void insert(String value, int pos) {
      insert(value.toCharArray(), pos);
   }

   public void insert(StringBuffer value, int pos) {
      insert(value.toString().toCharArray(), pos);
   }

   public void insert(ByteBuffer value, int pos) {
      insert(value.toByteArray(), pos);
   }

   public void insert(boolean value, int pos) {
      insert(String.valueOf(value), pos);
   }

   public void insert(char value, int pos) {
      if ( locked )
         return;
      if ( pos < 0 || pos >= length )
         throw new IndexOutOfBoundsException();
      checkCapacity(length+1);
      byte[] temp = new byte[length-pos];
      System.arraycopy(buffer, pos, temp, 0, temp.length);
      System.arraycopy(temp, 0, buffer, pos+1, temp.length);
      buffer[pos] = (byte)value;
      length++;
   }

   public void insert(byte value, int pos) {
      if ( locked )
         return;
      if ( pos < 0 || pos >= length )
         throw new IndexOutOfBoundsException();
      checkCapacity(length+1);
      byte[] temp = new byte[length-pos];
      System.arraycopy(buffer, pos, temp, 0, temp.length);
      System.arraycopy(temp, 0, buffer, pos+1, temp.length);
      buffer[pos] = value;
      length++;
   }

   public void insert(double value, int pos) {
      insert(String.valueOf(value), pos);
   }

   public void insert(float value, int pos) {
      insert(String.valueOf(value), pos);
   }

   public void insert(int value, int pos) {
      insert(String.valueOf(value), pos);
   }

   public void insert(long value, int pos) {
      insert(String.valueOf(value), pos);
   }

   public void insert(Object value, int pos) {
      insert(String.valueOf(value), pos);
   }

   public void delete(int pos, int width) {
      if ( locked )
         return;
      if ( pos+width > length )
         width = (length-pos);
      byte[] temp = new byte[length-(pos+width)];
      System.arraycopy(buffer, pos+width, temp, 0, temp.length);
      System.arraycopy(temp, 0, buffer, pos, temp.length);
      length -= width;
   }

   public void replace(byte[] value, int pos, int width) {
      delete(pos, width);
      insert(value, pos);
   }

   public void replace(String value, int pos, int width) {
      replace(value.getBytes(), pos, width);
   }

   public void replace(StringBuffer value, int pos, int width) {
      replace(value.toString().getBytes(), pos, width);
   }

   public void replace(ByteBuffer value, int pos, int width) {
      replace(value.toByteArray(), pos, width);
   }

   public void replace(boolean value, int pos, int width) {
      replace(String.valueOf(value), pos, width);
   }

   public void replace(char value, int pos, int width) {
      replace(String.valueOf(value), pos, width);
   }

   public void replace(double value, int pos, int width) {
      replace(String.valueOf(value), pos, width);
   }

   public void replace(float value, int pos, int width) {
      replace(String.valueOf(value), pos, width);
   }

   public void replace(int value, int pos, int width) {
      replace(String.valueOf(value), pos, width);
   }

   public void replace(long value, int pos, int width) {
      replace(String.valueOf(value), pos, width);
   }

   public void replace(Object value, int pos, int width) {
      replace(String.valueOf(value), pos, width);
   }

   public byte[] toByteArray() {
      byte[] value = new byte[length];
      System.arraycopy(buffer, 0, value, 0, length);
      return value;
   }

   public String toString() {
      return new String(buffer, 0, length);
   }

   public int length() {
      return length;
   }

   public void setLength(int value) {
      if ( locked )
         return;
      if ( value < 0 || value > length )
         throw new IndexOutOfBoundsException();
      length = value;
   }

   /* ByteArrayOutputStream style methods */

   public int size() {
      return length;
   }

   public void reset() {
      if ( locked )
         return;
      length = 0;
   }

   public byte byteAt(int index) {
      if ( index < 0 || index >= length )
         throw new IndexOutOfBoundsException();
      return buffer[index];
   }

   public void setByteAt(int index, byte value) {
      if ( locked )
         return;
      if ( index < 0 || index >= length )
         throw new IndexOutOfBoundsException();
      buffer[index] = value;
   }

   public void reverse() {
      if ( locked )
         return;
      byte b = 0;
      for ( int i = 0; i < length/2; i++ ) {
         b = buffer[i];
         buffer[i] = buffer[(length-1)-i];
         buffer[(length-1)-i] = b;
      }
   }

   public InputStream getInputStream() {
      return new ByteArrayInputStream(buffer, 0, length);
   }

   public void writeTo(OutputStream out) throws IOException {
      out.write(buffer, 0, length);
   }

   public void chunkTo(OutputStream out, int chunkSize) throws IOException {
      int pos = 0;
      int size = 0;
      while ( pos < length ) {
         size = length-pos > chunkSize ? chunkSize
                                       : length-pos;
         out.write(buffer, pos, size);
         pos += size;
         Thread.yield();
      }
   }

   public void chunkTo(OutputStream out) throws IOException {
      chunkTo(out, 2048);
   }

   public void readFrom(InputStream in) throws IOException {
      if ( locked )
         throw new IOException("ByteBuffer Is Locked");
      int size = in.available();
      checkCapacity(length+size);
      in.read(buffer, length, size);
      length += size;
   }

   public void write(int b) throws IOException {
      if ( locked )
         throw new IOException("ByteBuffer Is Locked");
      checkCapacity(length+1);
      buffer[length++] = (byte)b;
   }

   public void write(byte b[]) throws IOException {
      if ( locked )
         throw new IOException("ByteBuffer Is Locked");
      checkCapacity(length+b.length);
      System.arraycopy(b, 0, buffer, length, b.length);
      length += b.length;
   }

   public void write(byte b[], int off, int len) throws IOException {
      if ( locked )
         throw new IOException("ByteBuffer Is Locked");
      checkCapacity(length+len);
      System.arraycopy(b, off, buffer, length, len);
      length += len;
   }

   /* Serialization-Related Methods */

   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
      // Read the array in and set the length correctly
      in.defaultReadObject();
      length = buffer.length;
   }

   private void writeObject(ObjectOutputStream out) throws IOException {
      // Pack the array, so we're not saving crap
      pack();
      out.defaultWriteObject();
   }

   /* Buffer locking related methods */

   public void lock() {
      if ( !locked ) {
         pack();
         locked = true;
      }
   }

   public boolean isLocked() {
      return locked;
   }
}

